@@ -9,6 +9,7 @@ use tokio_tungstenite::{MaybeTlsStream, WebSocketStream, connect_async};
 use tokio_tungstenite::tungstenite::{Message};
 use tokio::net::TcpStream;
 use futures::{SinkExt, StreamExt};
+use futures_util::stream::{SplitSink, SplitStream};
 
 
 pub mod keys{
@@ -49,7 +50,7 @@ impl Api{
                 let (down_sender, down_receiver) = channel(buffer);
                 match connect_async(url).await{
                     Ok(conn) => {
-                        tokio::spawn(socket_listener(conn.0, up_sender, down_receiver));
+                        listen(conn.0, up_sender, down_receiver).await;
                         self.sender = Some(down_sender);
                         self.receiver = Some(up_receiver);
                         Ok(())
@@ -120,42 +121,50 @@ impl Api{
     }
 }
 
-async fn socket_listener(mut socket: WebSocketStream<MaybeTlsStream<TcpStream>>, upstream_sender: Sender<Vec<u8>>, mut downstream_receiver: Receiver<Vec<u8>>){
+async fn listen(socket: WebSocketStream<MaybeTlsStream<TcpStream>>, upstream_sender: Sender<Vec<u8>>, downstream_receiver: Receiver<Vec<u8>>){
+    let (s, r) = socket.split();
+    tokio::spawn(receiver_coro(s, downstream_receiver));
+    tokio::spawn(sender_coro(r, upstream_sender));
+    //tokio::spawn(socket_listener(socket, upstream_sender, downstream_receiver));
+}
+
+async fn receiver_coro(mut sender: SplitSink<WebSocketStream<MaybeTlsStream<TcpStream>>, Message>, mut downstream_receiver: Receiver<Vec<u8>>){
     loop {
-        tokio::select!{
-            inc = socket.next() => {
-                match inc{
-                    None => { break }
-                    Some(n) => {
-                        match n{
-                            Ok(msg) => {
-                                match msg{
-                                    Message::Binary(bin) => {
-                                        if let Err(_) =  upstream_sender.send(bin).await{ break }
-                                    }
-                                    _ => {}
-                                }
-                            }
-                            Err(_) => { break }
-                        }
-                    }
-                }
-            }
-            outc = downstream_receiver.recv() => {
-                match outc{
-                    None => { break }
-                    Some(bin) => {
-                        if let Err(_) = socket.send(Message::Binary(bin)).await{ break }
-                    }
+        match downstream_receiver.recv().await{
+            None => { break }
+            Some(bytes) => {
+                if let Err(_) = sender.send(Message::Binary(bytes)).await{
+                    break
                 }
             }
         }
     }
     downstream_receiver.close();
-    match socket.close(None).await{
+    match sender.close().await{
         Ok(_) => {}
         Err(_) => {}
-    };
+    }
+}
+
+async fn sender_coro(mut receiver: SplitStream<WebSocketStream<MaybeTlsStream<TcpStream>>>, upstream_sender: Sender<Vec<u8>>){
+    loop {
+        match receiver.next().await{
+            None => { break }
+            Some(bytes) => {
+                match bytes{
+                    Ok(bytes) => {
+                        match bytes {
+                            Message::Binary(bytes) => {
+                                if let Err(_) = upstream_sender.send(bytes).await{ break }
+                            }
+                            _ => {}
+                        }
+                    }
+                    Err(_) => { break }
+                }
+            }
+        }
+    }
 }
 
 impl Into<protocol::Package> for Msg{
